/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "source_histo.hh"
#include "frame_source.hh"
#include "chan_list.hh"

using namespace sends;
using namespace std;

//======================================  Source histo constructor
source_histo::source_histo(void) {
}

//======================================  Source histo destructor
source_histo::~source_histo(void) {
    clear();
}

//======================================  Bump count for all channel sources
void 
source_histo::bump(const chan_entry& chan, gps_type t) {
    index_type N = chan.getNSource();
    for (index_type i=0; i<N; ++i) {
	if (chan.source(i).test_gps(t)) bump(chan.source(i).name(), t);
    }
}

//======================================  Bump source count
void 
source_histo::bump(const string& name, gps_type t) {
    if (is_vetoed(name)) return;
    index_type inx = find(name);
    if (inx == size()) {
	source_info si(name);
	si._loc = Source_List.list_ptr(name, t);
	if (!si._loc) return;
	si._arch = si._loc->archive();
	av.push_back(si);
    }
    av[inx]._nchn++;
    while (inx && av[inx-1] < av[inx]) {
	av[inx].swap(av[inx-1]);
	inx--;
    }
}

//======================================  Clear source histo
void 
source_histo::clear(void) {
    index_type N = av.size();
    for (index_type i=0; i<N; ++i) {
	av[i].release();
    }
    av.clear();
}

//======================================  Fund the specified 
index_type
source_histo::find(const std::string& name) const {
    index_type N = size();
    for (index_type i=0; i<N; ++i) {
	if (av[i]._name == name) return i;
    }
    return N;
}

//======================================  Insert a type to the veto list.
void 
source_histo::veto(const std::string& name) {
    uv.insert(name);
}

//======================================  Release frame_loc
void
source_info::release(void) {
    //_loc->unlock();
    _loc = 0;
}

//======================================  Swap data two source_info instances
void
source_info::swap(source_info& x) {
    _name.swap(x._name);
    std::swap(_nchn, x._nchn);
    std::swap(_arch, x._arch);
    std::swap(_loc,  x._loc);
}
